;;; mode-icons.el --- Show icons for modes -*- lexical-binding: t; -*-

;; Copyright (C) 2012  Tom Willemsen

;; Author: Tom Willemsen <thomas@aethon.nl>
;; Keywords: multimedia
;; Version: 0.1.0

;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.

;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.

;;; Commentary:

;;

;;; Code:

(defconst mode-icons--directory
  (if load-file-name
      (file-name-directory load-file-name)
    default-directory)
  "Where mode-icons was loaded from.")

(defun mode-icons-get-icon-file (icon)
  "Get the location of ICON.

ICON should be a file name with extension.  The result is the
absolute path to ICON."
  (concat mode-icons--directory "/icons/" icon))

(defvar mode-icons
  `(("Emacs-Lisp" "emacs" xpm)
    ("Python" "python" xpm)
    ("Scheme" "scheme" xpm)
    ("Lisp" "cl" xpm)
    ("PHP" "php" xpm)
    ("HTML" "html" xpm)
    ("Org" "org" xpm))
  "Icons for major modes.

Each specification is a list with the first element being the
name of the major mode.  The second the name of the icon file,
without the extension.  And the third being the type of icon.")

(defun get-icon-display (icon type)
  "Get the value for the display property of ICON having TYPE.

ICON should be a string naming the file of the icon, without its
extension.  Type should be a symbol designating the file type for
the icon."
  (let ((icon-path (mode-icons-get-icon-file
                    (concat icon "." (symbol-name type)))))
   `(image :type ,type :file ,icon-path :ascent center)))

(defun propertize-mode (mode icon-spec)
  "Propertize MODE with ICON-SPEC.

MODE should be a string, the name of the mode to propertize.
ICON-SPEC should be a specification from `mode-icons'."
  (propertize
   mode 'display (get-icon-display (nth 1 icon-spec) (nth 2 icon-spec))))

(defun get-mode-icon (mode)
  "Get the icon for MODE, if there is one."
  (let* ((mode-name (format-mode-line mode))
         (icon-spec (assoc mode-name mode-icons)))
    (if icon-spec
        (propertize-mode mode-name icon-spec)
      mode-name)))

(defun set-mode-icon (mode)
  "Set the icon for MODE."
  (setq mode-name (get-mode-icon mode)))

;;;###autoload
(defun set-current-mode-icon ()
  "Set the icon for the current major mode."
  (set-mode-icon mode-name))

(define-minor-mode mode-icons-mode
  "Replace the name of the current major mode with an icon."
  :global t
  (if mode-icons-mode
      (add-hook 'after-change-major-mode-hook 'set-current-mode-icon)
    (remove-hook 'after-change-major-mode-hook 'set-current-mode-icon)))

(provide 'mode-icons)
;;; mode-icons.el ends here
